unit kstring;
{ This file includes string-handling subroutines
  Author:  Vesa Lappalainen
  Date:    1996-1997

  Additions:
    28.3.1997:  Split
    27.12.1997: GetIniName serches first current directory
    09.03.2000: Separate
    18.07.2000: Separate by string
                Split3 & 4
                CountDelimiters
                FindFirstDelimiter
}

interface
uses Classes, SysUtils;

type IdentifierSet = set of char;

function FindFirstDelimiter(const Delim,s:string):integer;
function FindLastDelimiter(const Delim,s:string):integer;
function CountDelimiters(const Delim,s:string):integer;
function Separate(var s:string; const ByWhat:char):string; overload;
function SeparateLast(var s:string; const ByWhat:string):string; overload;
function Separate(var s:string; const ByWhat:string):string; overload;
function Split(const s:string; const ByWhat:string; var parts:TStringList):integer;
function Split2(const s:string; const ByWhat:string; var part1,part2:string):integer;
function Split3(const s:string; const ByWhat:string; var part1,part2,part3:string):integer;
function Split4(const s:string; const ByWhat:string; var part1,part2,part3,part4:string):integer;
function Split2Trim(const s:string; const ByWhat:string; var part1,part2:string):integer;
function Split3Trim(const s:string; const ByWhat:string; var part1,part2,part3:string):integer;
function Split4Trim(const s:string; const ByWhat:string; var part1,part2,part3,part4:string):integer;
//function GetIniName(const n:string):string;
function ChangeExtension(const n,ext:string):string;
function ChangeAll(const s,swhat,sto:string):string;
function ValidIdentifier(const s:string;const Illegal,NotBeg:IdentifierSet):boolean;
function RemoveChars(const s,what:string) : string;
function wildmat(const mask:string; const sword:string): boolean;
function wildmatmU(const mask,sword:string):boolean;
function wildmatUU(const mask,sword:string):boolean;
function GetEnvVariable(const varname:string):string;

function LastNum(var s:string; n:integer):integer;
function FirstNum(var s:string; n:integer):integer;

function NowToStr:string;
function NowTo4Str:string;


implementation

{$ifdef DELPHI5}
uses Windows;
{$endif}

const MAXPARTS = 500;

{------------------------------------------------------------------------------}
{ CountDelimiters - counts how many Delim char is in s                         }
{------------------------------------------------------------------------------}
function CountDelimiters(const Delim,s:string):integer;
var i:integer;
begin
  Result := 0;
  for i:=1 to Length(s) do
    if ( IsDelimiter(Delim,s,i) ) then
      inc(Result);
end;

{------------------------------------------------------------------------------}
{ FindFirstDelimiter - like Pos, but finds first char in set Delim             }
{------------------------------------------------------------------------------}
function FindFirstDelimiter(const Delim,s:string):integer;
var i:integer;
begin
  Result := 0;
  for i:=1 to Length(s) do
    if ( IsDelimiter(Delim,s,i) ) then begin
      Result := i;
      Exit;
    end;
end;

function FindLastDelimiter(const Delim,s:string):integer;
var i:integer;
begin
  Result := 0;
  for i:=Length(s) downto 1 do
    if ( IsDelimiter(Delim,s,i) ) then begin
      Result := i;
      Exit;
    end;
end;



{------------------------------------------------------------------------------}
{ Split  - splits s separated by ByWhat to parts, returns number of parts      }
{------------------------------------------------------------------------------}
function Split(const s:string; const ByWhat:string; var parts:TStringList):integer;
{
  s = 'st1|st2,st3'  ByWhat = '|,'  =>  'st1'      'st2'      'st3'
  s = 'st1 st2,st3'  ByWhat = '|,'  =>  'st1 st2'  ''         'st3'
  s = 'st1|st2 st3'  ByWhat = '|,'  =>  'st1'      'st2 st3'  ''
  s = 'st1 st2 st3'  ByWhat = '|,'  =>  'st1 st2 st3'
  s = 'st1,st2,st3'  ByWhat = ','   =>  'st1'      'st2'      'st3'
  s = 'st1,,st3'     ByWhat = ','   =>  'st1'      ''         'st3'

  The last ByWhat-separator is copied until not found any more.
  Default is ; (when ByWhat='')
}
var by:char; st:string; ladd,a,lb,p,i:integer;
begin
  by := ';';
  lb := Length(ByWhat);

  st := s;
  if parts = nil then parts := TStringList.Create;
  if ( parts = nil ) then begin Result := -1; exit; end;
  parts.Clear;
  ladd := 0;

  for i:=0 to MAXPARTS do begin
    if ( i < lb ) then by := ByWhat[i+1];
    p := Pos(by,st);
    if ( p <= 0 ) then begin
      if ( i < lb ) then begin continue; end;
      parts.add(st);
      for a:=parts.Count+1 to lb+1 do parts.add('');
      Result := parts.Count;
      exit;
    end;
    parts.add(copy(st,1,p-1));
    for a:=ladd+1 to i do parts.add('');
    ladd := i+1;
    delete(st,1,p);
  end;
  Result := MAXPARTS+1;
end;

function Split2(const s:string; const ByWhat:string; var part1,part2:string):integer;
var p : integer;
begin
  Result := 2;
  p := FindFirstDelimiter(ByWhat,s); if p = 0 then begin p := 10000; Result := 1; end;
  part1 := copy(s,1,p-1);
  part2 := copy(s,p+1,10000);

end;

function Split3(const s:string; const ByWhat:string; var part1,part2,part3:string):integer;
var st : string;
begin
  st := s;
  Result := CountDelimiters(ByWhat,s)+1;
  part1 := Separate(st,ByWhat);
  part2 := Separate(st,ByWhat);
  part3 := Separate(st,ByWhat);
end;

function Split4(const s:string; const ByWhat:string; var part1,part2,part3,part4:string):integer;
var st : string;
begin
  st := s;
  Result := CountDelimiters(ByWhat,s)+1;
  part1 := Separate(st,ByWhat);
  part2 := Separate(st,ByWhat);
  part3 := Separate(st,ByWhat);
  part4 := Separate(st,ByWhat);
end;

function Split2Trim(const s:string; const ByWhat:string; var part1,part2:string):integer;
begin
  Result := Split2(s,ByWhat,part1,part2);
  part1 := Trim(part1);
  part2 := Trim(part2);

end;

function Split3Trim(const s:string; const ByWhat:string; var part1,part2,part3:string):integer;
begin
  Result := Split3(s,ByWhat,part1,part2,part3);
  part1 := Trim(part1);
  part2 := Trim(part2);
  part3 := Trim(part3);
end;

function Split4Trim(const s:string; const ByWhat:string; var part1,part2,part3,part4:string):integer;
begin
  Result := Split4(s,ByWhat,part1,part2,part3,part4);
  part1 := Trim(part1);
  part2 := Trim(part2);
  part3 := Trim(part3);
  part4 := Trim(part4);
end;

{------------------------------------------------------------------------------}
{ ChangeExtension - changes filenames extension                                }
{------------------------------------------------------------------------------}
function ChangeExtension(const n,ext:string):string;
begin
  Result := ChangeFileExt(n,ext);
end;

function ChangeAll(const s,swhat,sto:string):string;
var lw,lt,p,pl:integer; // n:integer;
begin
  Result := s;
//  n := 0;
  lw := Length(swhat);
  lt := Length(sto);
  pl := 0;
  while ( true ) do begin
    p := pos(swhat,Result);
    if ( p = 0 ) then Exit;
    if ( p = pl ) and ( lw <= lt ) then exit;  // Jos aaa:ssa a->aa
    delete(Result,p,lw);
    insert(sto,Result,p);
//    inc(n);
    pl := p;
  end;
end;

{------------------------------------------------------------------------------}
{ ValidIdentifier - checks if string si a valid identifier                     }
{------------------------------------------------------------------------------}
function ValidIdentifier(const s:string;const Illegal,NotBeg:IdentifierSet):boolean;
var i:integer;
begin
  Result := false;
  if ( s = '' ) then exit;
  if ( s[1] in Illegal ) or ( s[1] in  NotBeg ) then exit;  // Nopeampi kuin s1+s2
  for i:=2 to Length(s) do
    if ( s[i] in Illegal ) then exit;
//  if ( pos(' ',s) > 0 ) then exit;
  Result := true;
end;

function RemoveChars(const s,what:string) : string;
var i:integer;
begin
  Result := '';
  for i:=1 to Length(s) do begin
    if ( pos(s[i],what) = 0 ) then
      Result := Result + s[i];
  end;
end;

{------------------------------------------------------------------------------}
{ Separate - separates first part of s separetd by ByWhat                      }
{   s = 'one|two|three' , ByWhat = '|'   => Result := 'one', s := 'two|three'  }
{   s = 'one'           , ByWhat = '|'   => Result := 'one', s := ''           }
{   s = ''              , ByWhat = '|'   => Result := '',    s := ''           }
{   s = 'abc'           , ByWhat = ''    => Result := 'abc', s := ''           }
{------------------------------------------------------------------------------}
function Separate(var s:string; const ByWhat:char):string;
var p:integer;
begin
  p := pos(ByWhat,s);
  if ( p = 0 ) then begin Result := s; s := ''; exit; end;
  Result := copy(s,1,p-1);
  Delete(s,1,p);
end;


function Separate(var s:string; const ByWhat:string):string;
var p:integer;
begin
  if ( ByWhat = '' ) then begin Result := s; s := ''; exit; end;

  p := FindFirstDelimiter(ByWhat,s);
  if ( p = 0 ) then begin Result := s; s := ''; exit; end;
  Result := copy(s,1,p-1);
  Delete(s,1,p);
end;

function SeparateLast(var s:string; const ByWhat:string):string; overload;
var p:integer;
begin
  if ( ByWhat = '' ) then begin Result := s; s := ''; exit; end;

  p := FindLastDelimiter(ByWhat,s);
  if ( p = 0 ) then begin Result := s; s := ''; exit; end;
  Result := copy(s,1,p-1);
  Delete(s,1,p);
end;


{****************************************************************************}
function wildmat(
  const mask:string;              { s   Verrattava sana, jossa jok.}
  const sword:string              { s   Sana johon verrataan.      }
  )                       : boolean;
{  Funktiolla tutkitaan tsmk annettu mahd. jokereita sisltv
   maski sanaan.
     ? vastaa mit tahansa yht merkki.
     * vastaa 0 tai useampaa mit tahansa merkki.

    Muuttuu:    tasmaako_jokeriin
    Kutsuu:     -
    Tekij:     Vesa Lappalainen
    Pvm:        17.11.1990
    Algoritmi:  Kutsuu rekursiivista aliohjelmaa tasmaa
                Rekursiotasojen lukumr on * lukumr.

-----------------------------------------------------------------------------}
var m_pit,s_pit:integer;

  function tasmaa(
       m,                                   { s   Indeksi maskissa.          }
       s                  :integer          { s   Indeksi sanassa.           }
    )                     :boolean;
  {
    Sytt:     maski, sana (aliohjelman globaali)
    Muuttuu:    tasmaa
    Kutsuu:     -
    Tekij:     Vesa Lappalainen (matkien PD C-ohjelmaa)
    Pvm:        17.11.1990
    Algoritmi: Ilman * homma selv:
                 Edetn kumpaankin jonoa.  Tsm, mikli
                 Kummassakin jonossa aina vastaava merkki ja maskissa
                 ? ja jonot loppuvat yhtaikaa!
               Jos * ja maski loppuu, niin tsm.
               Jos * jlkeen jotakin, niin jotakin tytyy lyty
               sanan lopusta.  Siis on helpointa tehd kutsua
               tasmaa(jotakin,sanan_loppu) kunnes joko sana loppuu (ei)
               tai tsm sanan_loppuun (juu).

    Esimerkkej:
            1. Helppo tapaus:
               sana : Kissa
               maski: K?s?a

            2. Yksi * lopussa:
               sana : Kissa ja koira
               maski: K?s*    => tsm kun * lopussa.

            3. Yksi * keskell:
               sana : Kissa ja koira
               maski: K?s*a   => kokeillaan jonoon 'a' sanoja:
                                   sa ja koira  (ei)
                                    a ja koira  (ei)
                                    ...
                                             a  (juu)
                                 kunnes tsm tai sana loppuu.
            4. Kaksi thte keskell:
               sana : Kissa ja koira
               maski: K?s*a*a => kokeillaan jonoon 'a*a' sanoja:
                                   sa ja koira  (ei)
                                    a ja koira  => (1 thti keskell)
               sana : a ja koira
               maski: a*a     => kokeillaan jonoon 'a' sanoja
                                      ja koira  (ei)
                                      ja koira  (ei)
                                    ...
                                             a  (juu)
                                 kunnes tsm tai sana loppuu.
  }
  var c:char;
  begin { tasmaa }
    Result:=false;
    while (m<=m_pit) do begin                 { Kunnes maski loppuu.         }
      c:=mask[m]; m:=m+1;                     { Seuraava merkki maskista.    }
      if (c='*') then begin                   { Jos *.                       }
        if (m<=m_pit) then                    { Jollei maski loppu.          }
          while not tasmaa(m,s) do begin      { Etsitn mask.lop. sanan.lop.}
            s:=s+1; if s>s_pit then exit;
           end;                               { Etsitn mask.lop. sanan.lop.}
        Result:=true; exit;
       end                                    { Jos *.                       }
      else if s>s_pit then exit               { Jos sana lop. ennen aikojaan.}
      else if (c='?') or (c=sword[s]) then s:=s+1{ Sama tai jokeri => edetn}
      else exit                               { Ei sama eik ? eik *!       }
     end;                                     { Kunnes maski loppuu.         }
    if (s>s_pit) then Result:=true;           { Jos loppuivat yhtaikaa.     }
  end;  { tasmaa }

begin { wildmat }
  m_pit := length(mask); s_pit := length(sword);
  Result := tasmaa(1,1);
end;  { wildmat }

function wildmatmU(const mask,sword:string):boolean;
var uword : string;
begin
  uword := UpperCase(sword);
  Result := wildmat(mask,uword);
end;

function wildmatUU(const mask,sword:string):boolean;
var umask:string;
begin
  umask := UpperCase(mask);
  Result := wildmatmU(umask,sword);
end;

function GetEnvVariable(const varname:string):string;
{$ifdef DELPHI5}
 var s:string;
begin
  SetLength(s,500);
  GetEnvironmentVariable(PChar(varname),PChar(s),500);
  SetLength(s,StrLen(PCHar(s)));
  Result := s;
end;
{$else}
begin
  Result := GetEnvironmentVariable(varname);
end;
{$endif}

function NowToStr:string;
var Hour,Min,Sec,MSec:Word; s:string;
begin
  DecodeTime(Time,Hour,Min,Sec,MSec);
  MSec := MSec div 10;
  s := IntToStr(MSec); if ( Length(s) < 2 ) then s := '0'+s;
  Result := FormatDateTime('yymmdd hhnnss',now)+'.'+s;
end;

function NowTo4Str:string;
var Hour,Min,Sec,MSec:Word; s:string;
begin
  DecodeTime(Time,Hour,Min,Sec,MSec);
  MSec := MSec div 10;
  s := IntToStr(MSec); if ( Length(s) < 2 ) then s := '0'+s;
  Result := FormatDateTime('yyyymmdd hhnnss',now)+'.'+s;
end;

function LastNum(var s:string; n:integer):integer;
var s1:string;
begin
  Result := 0;
  if ( n <= 0 ) then exit;
  if ( Length(s) >= n ) then begin
    s1 := Copy(s,Length(s)-n+1,n);
    Delete(s,Length(s)-n+1,n);
  end
  else begin
    s1 := s;
    s := '';
  end;
  Result := StrToIntDef(s1,0);
end;

function FirstNum(var s:string; n:integer):integer;
var s1:string;
begin
  Result := 0;
  if ( n <= 0 ) then exit;
  if ( Length(s) >= n ) then begin
    s1 := Copy(s,1,n);
    Delete(s,1,n);
  end
  else begin
    s1 := s;
    s := '';
  end;
  Result := StrToIntDef(s1,0);
end;


end.


